// A profile list window
//
// David Hayward 
// Developer Technical Support
// AppleLink: DEVSUPPORT
//
// Copyrite 1995, Apple Computer,Inc
//
// This file contains the menus routines for a profile list window.
// 
// 12/13/94	david	first cut


//#include <PictUtil.h>
#include <Files.h>
#include <Drag.h>
#include <Fonts.h>
#include <QDOffScreen.h>
#include <Resources.h>
#include <GXTypes.h>

#include "appGlobals.h"
#include "appMain.h"
#include "appMenus.h"
#include "appErrors.h"
#include "appAEvts.h"

#include "win.h"
#include "winTables.h"
#include "winProfList.h"
#include "winProfListGetSet.h"
#include "winProfile.h"
#include "winProfileGetSet.h"

#include "listsUtils.h"
#include "dragUtils.h"
#include "faux3DUtils.h"
#include "colorsyncUtils.h"
#include "resourceUtils.h"



/**\
|**| ==============================================================================
|**| PRIVATE DEFINES
|**| ==============================================================================
\**/
#define rProfListBaseID			4000
#define rProfListWindID			rProfListBaseID			// 'WIND' resource id
#define rProfListOSTID			rProfListBaseID			// 'WIND' resource id
#define kWindowWidthMin			300
#define kWindowHeightMin		40
#define kHeaderHeight			20
#define kCellHeight				16


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTION PROTOTYPES
|**| ==============================================================================
\**/
Boolean			CMSearchRecordsEqual		( CMSearchRecord a, CMSearchRecord b);
CMError			BuildProfList				( winHandle win ) ;
OSErr			DoCreateProfListWindow		( winHandle win ) ;
void			ProfListMakeSysProfileCmnd	( winHandle win ) ;

void			HandleClickInProfList		( ListHandle list, Boolean doubleClick ) ;
pascal OSErr	StartListDrag				( ListHandle list, EventRecord *theEvent) ;
OSErr			AddFlavors					( DragReference drag, ItemReference item, ListHandle list) ;

void			LocalToGlobalRect			( Rect *r ) ;
void			LocalToGlobalRgn			( RgnHandle rgn ) ;


/**\
|**| ==============================================================================
|**| PUBLIC FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
	winActivateProfList
 *------------------------------------------------------------------------------*
		This is a ActivateProcPtr for ProfList windows.
		Things, such as activating/deactivation controls should go here
		This ProcPtr is envoked by CallWinActivateProc() which is called by:
			DoActivateEvent() which dispaches activate events and
			DoOSEvent() which handles events such as Suspend/Resume.
\*------------------------------------------------------------------------------*/
void	winActivateProfList ( winHandle win, Boolean activating ) 
{
	LActivate( activating, GetProfListList(win) ) ;
}


/*------------------------------------------------------------------------------*\
	winResumeProfList
 *------------------------------------------------------------------------------*
		This is a ResumeProcPtr for ProfList windows.
		This ProcPtr is envoked by CallWinResumeProc() which is called by:
			DoOSEvent() which handles events such as Suspend/Resume.
\*------------------------------------------------------------------------------*/
void	winResumeProfList ( winHandle win, Boolean resuming ) 
{
	if (resuming)
		(void) BuildProfList ( win ) ;
}


/*------------------------------------------------------------------------------*\
	winUpdateProfList
 *------------------------------------------------------------------------------*
		This is a UpdateProcPtr for ProfList windows.  
		It handles all drawing into the window.
		For ProfList windows, this means that this routine is responible for
		drawing the grow box, the header area, and the list itself.
		This ProcPtr is envoked by CallWinUpdateProc() which is called by:
			DoUpdateEvent() which dispaches update events.
\*------------------------------------------------------------------------------*/
void	winUpdateProfList ( winHandle win, EventRecord *e )
{
	Rect			docRect ;
	Rect			headRect, headUpdateRect ;
	Rect			growRect, growUpdateRect ;
	Rect			contRect, contUpdateRect ;
	WindowRef		window = (WindowRef)e->message;
	Rect 			updateRect = ((**(((CGrafPtr)window)->visRgn)).rgnBBox) ;
	GrafPtr			savedPort ;
	
	GetPort( &savedPort ) ;	
	SetPort( (GrafPtr)window ) ;

	BeginUpdate( window ) ;


	docRect = GetWinRect( win ) ;		// check if ok?
	
// draw header	
	headRect = docRect ;
	headRect.bottom = kHeaderHeight ;
	headRect.bottom -= 3;
	if (SectRect(&headRect,&updateRect,&headUpdateRect))
	{
		faux3DFillRect ( &headRect ) ;
		ForeColor(whiteColor) ;
		MoveTo (headRect.left, headRect.bottom+1) ; LineTo (headRect.right, headRect.bottom+1) ;
		ForeColor(blackColor) ;
		MoveTo (headRect.left, headRect.bottom  ) ; LineTo (headRect.right, headRect.bottom  ) ;
		MoveTo (headRect.left, headRect.bottom+2) ; LineTo (headRect.right, headRect.bottom+2) ;
	}

// update controls
	DrawControls( window ) ;
	
// draw content
	TextSize(9) ;
	TextFace(0) ;
	TextFont(geneva) ;

	contRect = docRect ;
	contRect.top += kHeaderHeight ;
	contRect.bottom -= kScrollWidth ;
	contRect.right -= kScrollWidth ;
	if (SectRect(&contRect,&updateRect,&contUpdateRect))
	{
		ListHandle list ;
		list = GetProfListList( win ) ;
		LSetDrawingMode( true, list ) ;
		LUpdate( ((CGrafPtr)window)->visRgn, list) ;		// not sure this is the rgn
	}

// draw GrowIcon
	growRect = docRect ;
	growRect.top += kHeaderHeight ;
	if (SectRect(&growRect,&updateRect,&growUpdateRect))
	{
		RgnHandle saveRgn = NewRgn() ;
		GetClip( saveRgn ) ;
		ClipRect( &growRect ) ;
		DrawGrowIcon( window ) ;
		SetClip( saveRgn ) ;
		DisposeRgn( saveRgn ) ;
	}
	
// signal that we finished drawing
	EndUpdate( window ) ;
	SetPort( savedPort ) ;
}


/*------------------------------------------------------------------------------*\
	winClickProfList
 *------------------------------------------------------------------------------*
		This is a ClickProcPtr for ProfList windows.
		So far, all this contains is a crude handling of the scroll controls
		Other things, such as initiating a drag-and-drop should also go here
		This ProcPtr is envoked by CallWinClickProc() which is called by:
			DoMouseDownEvent() which dispaches mouse down events 
\*------------------------------------------------------------------------------*/
void winClickProfList ( winHandle win, EventRecord *e ) 
{
	WindowRef			window ;
	ListHandle			list ;
	Boolean				dblClk ;
	Boolean				dragStarted ;
	StartListDragUPP	upp ;
	
	window = (WindowRef) GetWinWindow ( win ) ;
	list = GetProfListList( win ) ;
	
	SetPort( (GrafPtr)window ) ;
	
	// check to see if click is in list
	
	upp = NewStartListDragProc(StartListDrag) ;
	dblClk = LClickDrag ( e, list, upp, &dragStarted ) ;
	DisposeRoutineDescriptor( upp ) ;
	
	if (!dragStarted)
		HandleClickInProfList( list, dblClk ) ;
}


/*------------------------------------------------------------------------------*\
	winResizeProfList
 *------------------------------------------------------------------------------*
		This is a ResizeProcPtr for ProfList windows.
		This routine is responsible for re-arranging the contents of the window if
		it is resized.  On entry to this routine, the new window size can be found
		in the document's window's portRect.  At the very least, this routine 
		should call SetWinRect() to put the new window size in the winHandle.
		So far, the only other thing this routine does is repostion the scroll controls
		and issue the appropriate update events so everything is re-drawn correctly.
		This ProcPtr is envoked by CallWinResizeProc() which is called by:
			DoMouseDownEvent() which dispaches mouse down events and
			DoCreatePictWindow() which initially creates the window.
\*------------------------------------------------------------------------------*/
void	winResizeProfList ( winHandle win ) 
{
	ProfListDataHdl	data ;
	WindowRef		window ;
	Rect			horzInval, vertInval ;
	Rect			oldDocRect, newDocRect ;
	

	oldDocRect = GetWinRect( win ) ;

	data = (ProfListDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return;
	
	window = (WindowRef) GetWinWindow ( win ) ;
	newDocRect = ((CGrafPtr)window)->portRect ;
	SetWinRect( win, newDocRect ) ;

	horzInval = vertInval = newDocRect ;
	horzInval.top  = MIN( oldDocRect.bottom, newDocRect.bottom) - kScrollWidth;
	vertInval.left = MIN( oldDocRect.right, newDocRect.right)   - kScrollWidth;
	EraseRect( &horzInval ) ;
	EraseRect( &vertInval ) ;
	
	LSize(	newDocRect.right - kScrollWidth,
			newDocRect.bottom - kHeaderHeight - kScrollWidth,
			GetProfListList(win) ) ;

	InvalRect( &horzInval ) ;
	InvalRect( &vertInval ) ;
	
	return ;
}


/*------------------------------------------------------------------------------*\
	winCloseProfList
 *------------------------------------------------------------------------------*
		This is a CloseProcPtr for ProfList windows.
		So far, all this does is call DisposeWinHandle()
		This win type doesn't need to be saved so ther is no reason to prompt the
		user to save the file here.
		This ProcPtr is envoked by CallWinCloseProc() which is called by:
			DoMouseDownEvent() which dispached click events (e.g. close box), and
			MenuProcPtrs which are called whenever menu events occur.
\*------------------------------------------------------------------------------*/
void	winCloseProfList ( winHandle win ) 
{
	DisposeWinHandle( win ) ;
}


/*------------------------------------------------------------------------------*\
	winMenuProfList
 *------------------------------------------------------------------------------*
		This is a MenuProcPtr for ProfList windows.
		This routine dispatches any menu commands that the window can handle
		to the appropriate function.
		This ProcPtr is envoked by CallWinMenuProc() which is called by:
			HandleMenuCommand() which dispatches all menu events.
\*------------------------------------------------------------------------------*/
void winMenuProfList ( winHandle win, long menuResult, Boolean *didit )
{
	short			menuID;
	short			menuItem;
	
	*didit = true ;
	menuID   = HiWrd(menuResult) ;
	menuItem = LoWrd(menuResult) ;
	switch ( menuID )
	{
		case mProfiles:
			switch ( menuItem )
			{
				case iSetDefaultProf:
					ProfListMakeSysProfileCmnd( win ) ;
					break ;
				default :
					*didit = false ;
					break ;
			}
			break;
		default :
			*didit = false ;
			break ;
	}
	HiliteMenu(0) ;		// Unhighlight whatever MenuSelect or MenuKey hilited
}


/*------------------------------------------------------------------------------*\
	winUpdateMenusProfList
 *------------------------------------------------------------------------------*
		This is a UpdateMenuProcPtr for ProfList windows.
		This routine enables any menu commands that the window can handle.
		This ProcPtr is envoked by CallWinUpdateMenusProc() which is called by:
			DoAppAdjustMenus() which dispatches all menu events.
\*------------------------------------------------------------------------------*/
void winUpdateMenusProfList ( winHandle win ) 
{
	MenuHandle		theMenu ;
	winHandle		frontWin;
	
	frontWin = GetFrontWindowWinHandle() ;
	
	if ( win == frontWin )
	{
		// do the file menu
		theMenu = GetMenuHandle ( mFile ) ;
	
			EnableItem ( theMenu, kWholeMenu ) ;
			EnableItem ( theMenu, iClose ) ;

		// do the profiles menu
		theMenu = GetMenuHandle ( mProfiles ) ;
		
			{
			Cell		cell;
			ListHandle	list;
			list = GetProfListList( frontWin ) ;
			if ( GetSingleSelect( &cell, list))			// if we can fine one selected item
			{
				EnableItem ( theMenu, kWholeMenu ) ;
				EnableItem ( theMenu, iSetDefaultProf ) ;
			}
			}
	}	
}


/*------------------------------------------------------------------------------*\
	winAllocProfList
 *------------------------------------------------------------------------------*
		This is a AllocProcPtr for ProfList windows.
		This routine is responsible for filling in all the needed fields of the
		winHandle structure.  This routine shouldn't be called directly.
		Instead, the code which needs to create a document of this type should
		call NewWinHandle(win,winAllocProfList) to envoke this function.
		This ProcPtr is envoked by NewWinHandle() which is called by:
			DoProfListCommand() which handles the Profile:CS List menu command.
\*------------------------------------------------------------------------------*/
OSErr winAllocProfList ( winHandle win )
{
	OSErr			err = noErr ;
	ProfListDataHdl	data ;
	
	// we need ColorSync 2.
	if (!CS2_available())
		return eWarnNoColorSync ;

	// set the window type
	SetWinType( win, kProfListType ) ;
	
	// allocate data handle
	data = (ProfListDataHdl)NewHandleClear( sizeof(ProfListDataRec) ) ;	
	if ( data == nil ) return MemError() ;
	SetWinData ( win, (Handle)data ) ;
	
	// stuff winHandle fields 
	SetWinActivateProc		( win,    (ActivateProcPtr) winActivateProfList ) ;
	SetWinResumeProc		( win,      (ResumeProcPtr) winResumeProfList ) ;
	SetWinUpdateProc		( win,      (UpdateProcPtr) winUpdateProfList ) ;
	SetWinClickProc			( win,       (ClickProcPtr) winClickProfList ) ;
	SetWinMenuProc			( win,        (MenuProcPtr) winMenuProfList ) ;
	SetWinUpdateMenusProc	( win, (UpdateMenusProcPtr) winUpdateMenusProfList ) ;
	SetWinResizeProc		( win,      (ResizeProcPtr) winResizeProfList ) ;
	SetWinNewProc			( win,         (NewProcPtr) winNewProfList ) ;
	SetWinCloseProc			( win,       (CloseProcPtr) winCloseProfList ) ;
	SetWinDisposeProc		( win,     (DisposeProcPtr) winDisposeProfList ) ;
	
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winNewProfList
 *------------------------------------------------------------------------------*
		This is a NewProcPtr for ProfList windows.
		This routine is responsible for building a new ProfList window after all the
		needed fields of the winHandle structure have been filled in by winAllocProfList.
		If an ProfList window is already open, then this routine will bring it
		to the front instead of opening a second window.
		This routine shouldn't be called directly.  Instead, the code which needs
		to create a document of this type should call NewWinHandle(win,winAllocProfList),
		and then CallWinNewProc(win) to envoke this function.
		This ProcPtr is envoked by CallWinNewProc() which is called by:
			DoProfListCommand() which handles the Profile:CS List menu command.
\*------------------------------------------------------------------------------*/
OSErr winNewProfList ( winHandle win )
{
	OSErr				err = noErr ;
	winHandle			existingWin ;
	OSType				subtype ;
	
	// see if we are already open
	
	subtype = GetWinSubtype( win ) ;
	if ( subtype!=kProfIDListSubtype )
	if ( FindWinHandle( nil, kProfListType, subtype, 1, 1, &existingWin ) == 1 )
	{
		// bring it to the front
		SelectWindow( GetWinWindow(existingWin) ) ;		
		return kWasAlreadyOpen ;
	}

	err = DoCreateProfListWindow( win ) ;
	if ( err ) return err ;
		
	return err ;	
}


/*------------------------------------------------------------------------------*\
	winDisposeProfList
 *------------------------------------------------------------------------------*
		This is a DisposeProcPtr for ProfList windows.
		This routine is responsible for disposing of any data that was allocated
		by the document and stored in the its data handle.
		This ProcPtr is envoked by CallWinDisposeProc() which is called by:
			DisposeWinHandle() which disposes of the entire winHandle.
\*------------------------------------------------------------------------------*/
void winDisposeProfList ( winHandle win )
{
	ProfListDataHdl		data ;
	ListHandle			list ;
	CMProfileSearchRef	search ;
	CMProfileIdentifierHdl id ;

	data = (ProfListDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return;
	
	list = GetProfListList ( win ) ;
	if ( list != nil )
		LDispose(list) ;
	
	search = GetProfListSearchResult( win ) ;
	if ( search != nil )
		CMDisposeProfileSearch(search) ;

	id = GetProfListIdentHdl( win ) ;
	if ( id != nil )
		DisposeHandle( (Handle)id ) ;

	// lastly, dispose of the data handle
	DisposeHandle( (Handle)data ) ;
}


/**\
|**| ==============================================================================
|**| PRIVATE FUNCTIONS
|**| ==============================================================================
\**/


/*------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------*/
Boolean CMSearchRecordsEqual( CMSearchRecord a, CMSearchRecord b)
{
	return (	( a.CMMType == b.CMMType ) &&
				( a.profileClass == b.profileClass ) &&
				( a.dataColorSpace == b.dataColorSpace ) &&
				( a.profileConnectionSpace == b.profileConnectionSpace ) &&
				( a.deviceManufacturer == b.deviceManufacturer ) &&
				( a.deviceModel == b.deviceModel ) &&
				( a.deviceAttributes[0] == b.deviceAttributes[0] ) &&
				( a.deviceAttributes[1] == b.deviceAttributes[1] ) &&
				( a.profileFlags == b.profileFlags ) &&
				( a.searchMask == b.searchMask ) &&
				( a.filter == b.filter ) );
}	


/*------------------------------------------------------------------------------*\
\*------------------------------------------------------------------------------*/
static CMError BuildProfList ( winHandle win )
{
	ListHandle			theList = nil ;
	unsigned long		count ;
	CMError				cmerr = noErr ;


//------------- search for all profiles
	{
		CMProfileSearchRef	searchRef ;
		OSType				profileClass;
		
		profileClass = GetWinSubtype( win );
		searchRef = GetProfListSearchResult( win );
		
		if (searchRef==nil)
		{
			CMSearchRecord searchSpec = {cmMatchAnyProfile,0,0,0,0,0,0,0,0,0};
			
			if ( profileClass==kProfIDListSubtype )
			{
				CMProfileIdentifierHdl profIDHdl ;
				profIDHdl = GetProfListIdentHdl( win ) ;
				if (!profIDHdl) cmerr = nilHandleErr ;
				if (!cmerr) HLock( (Handle)profIDHdl );
				if (!cmerr) cmerr = CMProfileIdentifierFolderSearch( *profIDHdl, &count, &searchRef ) ;
			}
			else if ( profileClass==nil )
				cmerr = CMNewProfileSearch( &searchSpec, nil, &count, &searchRef ) ;
			else 
			{
				searchSpec.searchMask = cmMatchProfileClass ;
				searchSpec.profileClass = profileClass ;
				cmerr = CMNewProfileSearch( &searchSpec, nil, &count, &searchRef ) ;
			}
			
			if (!cmerr) SetProfListSearchResult( win, searchRef ) ;
		}
		else
			cmerr = CMUpdateProfileSearch( searchRef, nil, &count) ;
	}
	
	if (!cmerr) 
		SetProfListCount( win, count ) ;
	
	
//------------- build a list
	if (!cmerr) 
	{

		theList = GetProfListList( win );
		if (theList==nil)
		{
			Rect		rView = {	kHeaderHeight,		0,
									kWindowHeightMin,	kWindowWidthMin-kScrollWidth } ;
			Rect		rDataBnds ;
			Point		cellSize = {kCellHeight, kWindowWidthMin-kScrollWidth} ;
			WindowRef	window ;
			
			window = (WindowRef) GetWinWindow ( win ) ;
			SetRect( &rDataBnds, 0,0,1,count ) ; /// +1
			theList = LNew(&rView,
							&rDataBnds,
							cellSize,
							0,						// proc ID
							window,
							false,					// draw now
							true,					// hasGrow
							false,					// scrollHoriz
							true ) ;				// scrollVert
		}
		else
		{
			LSetDrawingMode(false,theList);
			LDelRow(0,0,theList);
			(void) LAddRow(count,0,theList);
		}
		SetProfListList( win, theList ) ;
	}


//------------- 
	if (!cmerr) 
	{
		Point				theCell ;
		unsigned long		v ;
		short				dataLen ;
		Str255				name ;
		CMProfileRef		prof ;

		for ( v=1; v<=count; v++ )
		{
			cmerr = CMSearchGetIndProfile( GetProfListSearchResult(win), v, &prof) ;
			cmerr = GetProfName( prof, (StringPtr)name ) ;
			CMCloseProfile( prof ) ;
			SetPt(&theCell,0,v-1) ;
			dataLen = name[0] ;
			LSetCell( &(name[1]), dataLen, theCell, theList ) ;
		}
		LSetDrawingMode(true,theList);
//		LUpdate( (*theList)->port->visRgn, theList );
	}
	
//-------------
bail :
	return cmerr ;
}


/*------------------------------------------------------------------------------*\
	DoCreateProfListWindow
 *------------------------------------------------------------------------------*
		This routine actually creates the WindowRef for a winHandle.
		After creating the window, it:
			reference the window and the DocumentRecord to each other,
			makes the window the current port,
			loads and adds the scroll bar controls,
			resizes the window according to the #defined constants,
			makes it visible .
		This routine is called by:
			winOpenProfList() which fills in the winHandle structure
\*------------------------------------------------------------------------------*/
OSErr DoCreateProfListWindow ( winHandle win )
{

	Rect			sizeRect ;
	OSErr			err = noErr;
	WindowRef		window ;
	ProfListDataHdl	data ;
	short			h,v ;
	unsigned long	count ;
	OSType			subtype ;
	Str255			winTitle="\p" ;
	
	data = (ProfListDataHdl) GetWinData ( win ) ;
	if ( data==nil ) return nilHandleErr;
	
	// create the window
	window = GetNewCWindow(rProfListWindID, nil, (WindowRef)-1 ) ;

	SetWinWindow( win, window ) ;			// save a reference to the window in the DocumentRecord
	SetWindowWinHandle( window, win ) ;		// save a reference to the DocumentRecord in the window

	SetGWorld( (CGrafPtr)GetWinWindow(win), nil ) ;	// set the window to be the current port

	// set the name of this document window
	subtype = GetWinSubtype( win ) ;
	err = GetOSTypeListStringPtr( rProfListOSTID, subtype, winTitle ) ;
	if (!err) SetWTitle( window, winTitle ) ;

	// try to read the ColorSync Profiles Folder
	err = BuildProfList( win ) ;
	count = GetProfListCount( win ) + 1 ;	// add one for the "System Profile" at top of list
	
	h = kWindowWidthMin ;
	v = MIN(count,20) * kCellHeight ;
	v += kScrollWidth + kHeaderHeight;
	v = MAX(v, kWindowHeightMin+kHeaderHeight) ;
	
	SizeWindow(	window, h, v, false) ;		// we'll update everything later

	SetRect( &sizeRect, kWindowWidthMin,
					    kWindowHeightMin+kHeaderHeight,
					    h+1, v+1) ;
	SetWinSizeRect( win, sizeRect ) ;
	
	CallWinResizeProc( win ) ;
	
	// make sure it is visible
	ShowWindow( window ) ;
	
	return err ;
}


/*------------------------------------------------------------------------------*\
	ProfListMakeSysProfileCmnd
 *------------------------------------------------------------------------------*
\*------------------------------------------------------------------------------*/
void ProfListMakeSysProfileCmnd ( winHandle win ) 
{
	ListHandle			list ;
	Point				cell = {0,0} ;
	CMProfileSearchRef	search ;
	unsigned long		index ;
	OSErr				err ;
	FSSpec				spec ;
	
	list = GetProfListList( win ) ;
			
	if ( LGetSelect(true, &cell, list))
	{
		index = cell.v + 1 ;
		search = GetProfListSearchResult( win ) ;
		err = CMSearchGetIndProfileFileSpec( search, index, &spec ) ;
		if (!err) err = CMSetSystemProfile( &spec ) ;
	}
}


/*------------------------------------------------------------------*/

pascal OSErr StartListDrag ( ListHandle list, EventRecord *theEvent)
{
	OSErr			err ;
	DragReference	drag ;
	RgnHandle		dragRgn ;
	ItemReference	theItem = 1;

	err = NewDrag( &drag );

	if (err == noErr)
	{		
		err = AddFlavors( drag, theItem, list);
		
		if (err == noErr)
		{
			dragRgn = NewRgn() ;
			if ( GetListSelectionRgn( list, dragRgn ) == true)
			{
				LocalToGlobalRgn( dragRgn ) ;	
				OutlineRegion( dragRgn ) ;
				err = TrackDrag( drag, theEvent, dragRgn ) ;
			}
			DisposeRgn( dragRgn ) ;
		}
		DisposeDrag( drag ) ;
	}
	return err;
}


/*------------------------------------------------------------------*/

static void HandleClickInProfList ( ListHandle list, Boolean doubleClick )
{
	Boolean				notAtEnd = true ;
	Point				cell = {0,0} ;
	CMProfileSearchRef	search ;
	FSSpec				spec ;
	unsigned long		index ;
	OSErr				err ;
	winHandle			win ;
	WindowRef			window ;
	
	window = (WindowRef)((**list).port) ;
	win = GetWindowWinHandle( window ) ;
	search = GetProfListSearchResult( win ) ;

	if ( doubleClick )
	{
		while ( notAtEnd && LGetSelect(true,&cell,list) )
		{
			index = cell.v + 1 ;
			err = CMSearchGetIndProfileFileSpec( search, index, &spec) ;
			SendODOC( &spec ) ;
			notAtEnd = LNextCell( false, true, &cell, list ) ;
		}
	}
	DoAppAdjustMenus() ;		// undim app items
}



/*------------------------------------------------------------------*/

OSErr AddFlavors ( DragReference drag, ItemReference item, ListHandle list)
{
	Boolean				notAtEnd = true ;
	short				dataSize;
	Cell				cell = {0,0} ;
	HFSFlavor			theHFSFlavor ;
	Rect				dragBounds ;
	CMProfileSearchRef	search ;
	FSSpec				spec ;
	FInfo				fndrInfo ;
	unsigned long		index ;
	OSErr				err ;
	WindowRef			window ;
	winHandle			win ;
	
	dataSize = sizeof(HFSFlavor);
	window = (WindowRef)((**list).port) ;
	win = GetWindowWinHandle( window ) ;
	search = GetProfListSearchResult( win ) ;

	while ( notAtEnd && LGetSelect(true,&cell,list) )
	{
		index = cell.v + 1 ;
		
		err = CMSearchGetIndProfileFileSpec( search, index, &spec ) ;

		err = FSpGetFInfo(&spec,&fndrInfo);
			theHFSFlavor.fileSpec = spec ;
			theHFSFlavor.fileType = fndrInfo.fdType ;
			theHFSFlavor.fileCreator = fndrInfo.fdCreator ;
			theHFSFlavor.fdFlags = fndrInfo.fdFlags ;
	
		err = AddDragItemFlavor( drag, item, flavorTypeHFS, (Ptr)&theHFSFlavor, dataSize, flavorSenderOnly);

		GetCellRect( list, &cell, &dragBounds ) ;
		LocalToGlobalRect( &dragBounds ) ;	
		err = SetDragItemBounds( drag, item, &dragBounds ) ;
	
		item ++ ;

		notAtEnd = LNextCell( false, true, &cell, list ) ;
	}
	return err ;
}


/*------------------------------------------------------------------*/

static void LocalToGlobalRect(Rect *r)
{
	LocalToGlobal((Point*)(&r->top));
	LocalToGlobal((Point*)(&r->bottom));
}


/*------------------------------------------------------------------*/

static void LocalToGlobalRgn( RgnHandle rgn )
{
	Point	pt = {0,0} ;
	LocalToGlobal( &pt ) ;
	OffsetRgn( rgn, pt.h, pt.v ) ;
}



